%%%-------------------------------------------------------------------
%%% File    : lgi.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Law Governed Interaction
%%%
%%% Created : 27 Apr 2004 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id$
%%
%%**
%% 
%% @doc Law Governed Interaction
%% @end
%%*
-module(lgi).

-export([behaviour_info/1]).

-export([cid/1, cid/2, aid/2, aid/3, aid/4, connect/3, connect_virtual/3, disconnect/1, disconnect/2]).
-export([deliver/0, deliver/1, deliver/2, forward/0, forward/1, forward/2]).
-export([get@/1, rm@/1, send/3, cast/3, cast/4, set@/2, aid@/0]).
-export([start_link/0, stop/0]).

-import(lgi.agent).

-define(answer_timeout, 5000).

behaviour_info(callbacks) ->
  [{initial_context, 1}, {sent, 2}, {arrived, 2}, {post_condition, 4}].

start_link() ->
  agent:start_link().

stop() ->
  agent:stop().

%% Props = [Prop], Prop = {law_module, ModName} | {K, V}
%% @spec connect(Law::atom(), Name::term(), Props) -> {ok, aid()}
%% Props = [{Key, Value}]
%%
%% @doc Connect to law _Law under the name _Name_ and become the agent owner.
%% Only agent owner can stop agent.
connect(Law, Name, Props) ->
  agent:new(Law, Name, Props).

%% Props = [Prop], Prop = {law_module, ModName} | {K, V}
%% @spec connect_virtual(Law::atom(), Name::term(), Props) -> {ok, aid()}
%% Props = [{Key, Value}]
%%
%% @doc Create anonymous connection to law _Law under the name _Name_.
connect_virtual(Law, Name, Props) ->
  agent:new(Law, Name, undefined, Props).

disconnect(Aid) ->
  agent:delete(Aid).

disconnect(Law, Name) ->
  agent:delete(Law, Name).

%% @spec cast(From, To, Msg = term()) -> Result::term() | exit()
%% From = To  = cid() | aid()
%% 
cast(From, To, Msg) ->
  cast(From, To, Msg, ?answer_timeout).

cast(From, To, Msg, Timeout) -> 
  agent:cast(From, To, Msg, Timeout).

send(From, To, Msg) ->
  agent:send(From, To, Msg).

%% @spec aid(Law, Name) -> aid()
%% @doc Create agent id with _Name_ under _Law_
aid(Law, Name) ->
  aid(Law, Name, node()).

aid(Law, Name, Node) ->
  agent:aid(Law, undefined, Name, Node).

%% @spec cid(Aid::aid()) -> cid()
%% @doc Create client id for agent with _Aid_.
%% @see cast/3, send/3
cid(Aid) ->
  cid(self(), Aid).

cid(Pid, Aid) ->
  agent:cid(Pid, Aid).

aid(Law, Pid, Name, Node) ->
  agent:aid(Law, Pid, Name, Node).

aid@() ->
  agent:law_do(aid).

set@(K, V) ->
  agent:law_do({context_set, K, V}).

get@(K) ->
  agent:law_do({context_get, K}).

rm@(K) ->
  agent:law_do({context_delete, K}).

forward() ->
  agent:law_do(forward).

forward(To) ->
  agent:law_do({forward, To}).

forward(To, Msg) ->
  agent:law_do({forward, To, Msg}).

deliver() ->
  agent:law_do(deliver).

deliver(To) ->
  agent:law_do({deliver, To}).

deliver(To, Msg) ->
  agent:law_do({deliver, To, Msg}).
