%%% File    : wsite_acl.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Apr 2003 by  <svg@surnet.ru>

%%
%% $Id: wsite_acl.erl,v 1.1 2003/09/01 15:18:40 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsite_law, []).

-behaviour(lgi).

-export([initial_context/1, sent/2, arrived/2, post_condition/4]).

%% internal export
-export([authenticate/1, authorize/1, login/1, logout/1]).
-export([component_access/2, db_access/2]).

-import(wslib.passwd).
-import(wslib.props).
-import(wsite.db.security).

-include("wstl.hrl").

-record(state, {db}).

initial_context(Props) ->
  Props.

sent(_To, {Action, Data}) ->
  {ok, Db} = odbcdb:connect(),
  lgi:set@(db, Db),
  THIS:Action(Data).

arrived(_From, _Msg) ->
  lgi:deliver().

post_condition(sent, _, {login, {LoginId, _}}, true) ->
  Db = lgi:get@(db),
  security:inc_login_counter(Db, LoginId),
  odbcdb:disconnect(Db),
  ok;
post_condition(_, _, _, _) ->
  Db = lgi:get@(db),
  odbcdb:disconnect(Db),
  ok.

authenticate({LoginId, Password}) ->
  Db = lgi:get@(db),
  case security:get_login(Db, LoginId) of
    undefined ->
      reject(login_undefined);
    Login ->
      [Cipher, Enabled] = props:gv_map([password, enabled], Login),
      (Enabled == true) orelse reject(disabled),
      passwd:authenticate(ascii, Password, Cipher)
	orelse reject(bad_password),
      accept()
  end.

login(Data) ->
  authenticate(Data).

logout({_User, _Attrs}) ->
  ok.

authorize({User, Service, Data}) ->
  Groups = authority_groups(User, Service, Data),
  authorize(Service, Data, Groups++['ANONYMOUS']).

authorize(Service, Data, Groups) ->
  lists:foreach(fun (G) ->
		    THIS:Service(G, Data) andalso accept()
		end, Groups),
  reject(). % default

authority_groups(User, component_access, _Data) ->
  Db = lgi:get@(db),
  security:permission_groups(Db, User, 'COMPONENT_ACCESS');
authority_groups(User, db_access, {Permission, _}) ->
    Db = lgi:get@(db),
  security:permission_groups(Db, User, Permission).

%%
component_access(Group, {Event, Path, AccessId}) ->
  component_access(Group, Event, Path, AccessId).

component_access(_Group, _Event, _Path, logout) ->
  true;
component_access(_G, _E, ["approot"], _A) ->
  true;
component_access(_G, _E, ["index"], _A) ->
  true;
component_access('SUPER_OPERATOR', _E, _P, _A) ->
  true;
component_access('OPERATOR', _E, _P, _A) ->
  true;
component_access(_G, _E, _P, _A) ->
  false.

%%
db_access(Group, {Permission, Attrs}) ->
  db_access(Group, Permission, Attrs).

db_access('OPERATOR', 'ACCESS_OWN_ORGANIZATION', _) ->
  true;
db_access('SUPER_OPERATOR', 'ACCESS_ANY_ORGANIZATION', _) ->
  true;
db_access(_, _, _) ->
  false.

%%
accept() ->
  wsacl:accept().

reject() ->
  wsacl:reject().

reject(Reason) ->
  wsacl:reject(Reason).
