%%%-------------------------------------------------------------------
%%% File    : wsacl.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 30 Nov 2004 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsacl).
-author(svg@surnet.ru).

-export([login/3, logout/2, authorize/4, postload_config/2]).
%% law module exports
-export([accept/0, accept/1, reject/0, reject/1]).
%% internal exports
-export([store_config/2, postload_config/2, remove_config/1, init_env/1, update_env/2]).

-import(wslib.props).

-include("wstl.hrl").

-define(default_module, wstl_law).

login(WA, LoginId, Password) ->
  case apply_law(WA, {login, {LoginId, Password}}) of
    permit ->
      {ok, LoginId};
    {deny, Reason} ->
      {error, Reason};
    deny ->
      {error, deny}
  end.

logout(WA, User) ->
  apply_law(WA, {logout, User}).

authorize(WA, User, Service, Data) ->
  apply_law(WA, {authorize, {User, Service, Data}}).

apply_law(WA, Req) ->
  {Law, Agent} = get_env(WA),
  AgentId = lgi:aid(Law, Agent),
  ClientId = lgi:cid(AgentId),
  Ret =
    case lgi:cast(ClientId, ClientId, Req) of
      false ->
	{error, [], "law error", ""};
      true ->
	receive_reply(AgentId)
    end,
  Ret.

receive_reply(AgentId) ->
  receive
    {AgentId, Answer} ->
      Answer
  end.

%%
accept() ->
  reply(permit).

accept(Data) ->
  reply(permit, Data).

reject() ->
  reply(deny).

reject(Data) ->
  reply(deny, Data).

reply(Status) ->
  lgi:deliver(self, {lgi:aid@(), Status}),
  exit(normal).

reply(Status, Data) ->
  lgi:deliver(self, {lgi:aid@(), {Status, Data}}),
  exit(normal).

%%

store_config({?MODULE, Props}, Entries) ->
  AppName = props:gv(application, Entries),
  {ok, NewProps} = props:check_required([{agent, AppName},
					 {law, AppName},
					 {module, ?default_module}
					],
					Props),
  {?MODULE, NewProps};
store_config(eof, Entries) ->
  case proplists:is_defined(?MODULE, Entries) of
    true -> [];
    false -> store_config({?MODULE, []}, Entries)
  end.

postload_config(Config, _Sup) ->
  application:start(lgi), % be shure that it is started
  [Law, Agent, Module] =
    props:gv_map([law, agent, module], wstl_conf:gv(Config, ?MODULE)),
  {ok, _} = lgi:connect_virtual(Law, Agent, [{law_module, Module}]).

remove_config(Config) ->
  [Law, Agent] = props:gv_map([law, agent], wstl_conf:gv(Config, ?MODULE)),
  lgi:disconnect(Law, Agent),
  ok.

init_env(Config) ->
  [Law, Agent] = props:gv_map([law, agent], wstl_conf:gv(Config, ?MODULE)),
  {ok, {Law, Agent}}.

update_env(WA=#ws_arg{ws_env=WE}, State) ->
  {ok, WA#ws_arg{ws_env=[{?MODULE, State}|WE]}, State}.

get_env(#ws_arg{ws_env=WE}) ->
  props:gv(?MODULE, WE).
