%%% File    : wslanguage.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Localization support
%%% Created : 13 May 2003 by  <svg@surnet.ru>

%%
%% $Id: wslanguage.erl,v 1.2 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc Localization support
%% @end
%%*
-module(wslanguage).

-export([start_link/2, postload_config/2, store_config/2]).
-export([msg/2, msg/3, find_msg/3, charset/1, language/1]).
%%
-export([init_env/1, update_env/2]).

-import(wslib.http).
-import(wslib.props).

-include("wstl.hrl").

-define(default_props, [{src, [{wslanguage_phrases, all}]},
			{default_lang, en},
			{default_charset, 'iso-8859-1'}
		       ]).

-define(lang_arg, list_to_atom(?MODULE_STRING "_arg")).
-record(lang_arg, {server,
		   lang,
		   charset,
		   default_lang,
		   default_charset}).

msg(WA, Name) ->
  msg(WA, Name, []).

msg(WA, Name, Params) ->
  case msg_find(get_env(WA), Name) of
    undefined when is_list(Name) ->
      io_lib:format("~s", [Name]);
    undefined ->
      io_lib:write(Name);
    Fun ->
      Fun(Params)
  end.

find_msg(WA, Name, Params) ->
  case msg_find(get_env(WA), Name) of
    undefined ->
      undefined;
    Fun ->
      Fun(Params)
  end.

charset(WA) ->  
  (get_env(WA))#lang_arg.charset.

language(WA) ->  
  (get_env(WA))#lang_arg.lang.

start_link(N={_, Name}, Options) ->
  phrase.ets_dict:start_link(N, Name, Options).

store_config({?MODULE, Props}, _Entries) ->
  {ok, NewProps} = props:check_required(?default_props, Props),
  {?MODULE, NewProps};
store_config(eof, Entries) ->
  case proplists:is_defined(?MODULE, Entries) of
    true -> [];
    false -> store_config({?MODULE, []}, Entries)
  end.
  
postload_config(Config, Sup) ->
  [Src, DefLang, DefCharset] =
    props:gv_map(?default_props, wstl_conf:gv(Config, ?MODULE)),
  PhraseProps = [{src, Fun} || Fun <- Src],
  SrvName = wstl_conf:service_name(Config, ?MODULE),
  SupSpec =
    {?MODULE,
     {?MODULE, start_link, [{local, SrvName}, PhraseProps]},
     permanent, 5000, worker, [?MODULE]
    },
  {ok, _} = supervisor:start_child(Sup, SupSpec),
  LA = #lang_arg{server=SrvName,
		 default_lang=DefLang,
		 default_charset=DefCharset},
  wstl_conf:setv(Config, ?lang_arg, LA).

init_env(Config) ->
  LA = wstl_conf:gv(Config, ?lang_arg),
  {ok, LA}.

update_env(WA=#ws_arg{yaws_arg=YA, ws_env=WE},
	   LA=#lang_arg{server=S, default_charset=DC, default_lang=DL}) ->
  UserLangs = header_value('Accept-Language', YA, [DL]),
  {Lang, CharSet} = get_current_env(S, UserLangs, {DL, DC}),
  Env = LA#lang_arg{lang=Lang, charset=CharSet},
  {ok, WA#ws_arg{ws_env=[{?MODULE, Env}|WE]}, LA}.

get_env(#ws_arg{ws_env=WE}) ->
  case proplists:get_value(?MODULE, WE) of
    undefined ->
      exit({service_not_started, ?MODULE});
    V ->
      V
  end.

get_current_env(Serv, Langs, Def) ->
  case get_current_env(Serv, Langs) of
    undefined ->
      Def;
    Ret ->
      Ret
  end.

get_current_env(_Srv, []) ->
  undefined;
get_current_env(Srv, [Lang|Rest]) ->
  case catch phrase:msg(Srv, {accept_language, Lang}) of
    {'EXIT', _} ->
      get_current_env(Srv, Rest);
    MsgFun ->
      {CharSet, _, _} = MsgFun([]),
      {Lang, CharSet}
  end.

header_value(Name, Arg, Def) ->
  case http:header_value(Name, Arg) of
    undefined ->
      Def;
    V ->
      V
  end.

msg_find(_LA=#lang_arg{server=Srv, lang=L, default_lang=DL}, Name) ->
  msg_find(Srv, Name, msg_langs([L, DL])).

msg_find(_Srv, _Name, []) ->
  undefined;
msg_find(Srv, Name, [Lang|Rest]) ->
  case phrase:find_msg(Srv, {Lang, Name}) of
    undefined ->
      msg_find(Srv, Name, Rest);
    MsgFun ->
      MsgFun
  end.
      
msg_langs(Ls) ->
  lists:flatmap(fun (V={Lang, _Country}) ->
		    [V, Lang];
		    (Lang) ->
		    [Lang]
		end, Ls).
