%%%-------------------------------------------------------------------
%%% File    : wstl_law.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 26 Feb 2005 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wstl_law).
-author(svg@surnet.ru).

-behaviour(lgi).

-export([initial_context/1, sent/2, arrived/2, post_condition/4]).

%% internal export
-export([authenticate/1, authorize/1, login/1, logout/1]).
-export([component_access/2]).

-import(wslib.passwd).
-import(wslib.props).
-import(wsite.db.security).

-include("wstl.hrl").

-record(state, {db}).

initial_context(Props) ->
  Props.

sent(_To, {Action, Data}) ->
  ?MODULE:Action(Data).

arrived(_From, _Msg) ->
  lgi:deliver().

post_condition(sent, _, {login, {_LoginId, _Password}}, true) ->
  ok;
post_condition(_, _, _, _) ->
  ok.


login(Data) ->
  authenticate(Data).

logout(_User) ->
  accept().

authenticate({_LoginId, _Password}) ->
  accept().

authorize({User, Service, Data}) ->
  Groups = authority_groups(User, Service, Data),
  authorize(Service, Data, Groups++['ANONYMOUS']).

authorize(Service, Data, Groups) ->
  lists:foreach(fun (G) ->
		    ?MODULE:Service(G, Data) andalso accept()
		end, Groups),
  accept(). % default

authority_groups(_User, component_access, _Data) ->
  [].
%%
component_access(Group, {Event, Path, AccessId}) ->
  component_access(Group, Event, Path, AccessId).

component_access(_Group, _Event, _Path, logout) ->
  true;
component_access(_G, _E, ["approot"], _A) ->
  true;
component_access(_G, _E, ["index"], _A) ->
  true;
component_access(_G, _E, _P, _A) ->
  true.

%%
accept() ->
  wsacl:accept().

reject() ->
  wsacl:reject().

reject(Reason) ->
  wsacl:reject(Reason).
