%%%-------------------------------------------------------------------
%%% File    : multicount.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 28 Feb 2005 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(multicount).
-author(svg@surnet.ru).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-export([update/4]).

-include("wstl.hrl").

-define(default_props, [{secure, false}]).

-record(state, {cnt=0, ch_max=0}).

access_info(_WA, Srv, State) ->
  #access_info{events=[do_incr, do_decr, do_exit, do_clone],
	       default_event=refresh}.

%
init(Arg, Props) ->
  {ok, update, #state{}, Props++?default_props}.

handle_event(_, security_alert, WA, Srv, State, S) ->
  {stop, ok};
handle_event(Event, old_request, WA, Srv, State, S) ->
  update(refresh, WA, Srv, S);
handle_event(Msg, From, WA, Srv, State, S) ->
  update(refresh, WA, Srv, S).

update(refresh, WA, Srv, S=#state{cnt=Cnt}) ->
  Out = gen_out(Cnt, Srv),
  {next_state, ok, Out, update, S};

update(do_clone, WA, Srv, S=#state{cnt=Cnt, ch_max=ChMax}) ->
  Name = integer_to_list(ChMax+1),
  Comp = wsdcomp:create(?MODULE, WA, WA,
			[{template, "multicount"},
			 {place_at, Name}
			]),
  ok = wsdcomp:spawn_comp(Srv, Name, Comp),
  wsdcomp:call(Srv, Name, {set_counter, Cnt}),
  Out = gen_out(Cnt, Srv),
  {next_state, ok, Out, update, S#state{ch_max=ChMax+1}};

update(do_incr, WA, Srv, S=#state{cnt=Cnt}) ->
  Out = gen_out(Cnt+1, Srv),
  {next_state, ok, Out, update, S#state{cnt=Cnt+1}};

update(do_decr, WA, Srv, S=#state{cnt=Cnt}) when Cnt > 0 ->
  Out = gen_out(Cnt-1, Srv),
  {next_state, ok, Out, update, S#state{cnt=Cnt-1}};
update(do_decr, WA, Srv, S=#state{cnt=Cnt}) ->
  wslib:show_lerror('MUST BE >= 0', [], WA),
  Out = gen_out(Cnt, Srv),
  {next_state, ok, Out, update, S};

update(do_exit, WA, Srv, S=#state{cnt=Cnt}) ->
  Out = gen_out(Cnt, Srv),
  {stop, ok, Out};

update({set_counter, Cnt}, WA, Srv, S) ->
  Out = gen_out(Cnt, Srv),
  {next_state, ok, Out, update, S#state{cnt=Cnt}}.

gen_out(Cnt, Srv) ->
  {Dead, Alive} =
    lists:foldl(fun (C, {D, A}) ->
		    case wsdcomp:is_alive(Srv, C) of
		      true ->
			{D, [C|A]};
		      false ->
			{[C|D], A}
		    end
		end, {[], []}, wsdcomp:children(Srv)),
  wsdcomp:display(Srv, Dead, false),
  Out = [{'Counter', Cnt}, {'Children', lists:reverse(Alive)}].

